' ====================================================================
' File: date_generator.vb
' Author: Adrian Kosmaczewski, 2006
' This file is a VBScript program that generates the "dates.sql" file,
' used to fill the BirthdayCard.db SQLite database with random dates.
' ====================================================================


' ====================================================================
' Creates a random date between in a given range
'
' LowerDate: the minimum Date of the range
' UpperDate: the maximum Date of the range
' Returns:   a Long integer value
' ====================================================================
Function RandomDateInRange(LowerDate, UpperDate)
    RandomDateInRange = CLng((UpperDate - LowerDate + 1) * Rnd + LowerDate)
End Function

' ====================================================================
' This function transforms a Date object into a string
' compliant with the ISO 8601 standard ("YYYY-MM-DD") as described here:
' http://www.cl.cam.ac.uk/~mgk25/iso-time.html
'
' d: a Date object
' Returns: a String
' ====================================================================
Function DateToStandardString(d)
    Dim y, m, dd
    y = Year(d)
    m = Month(d)
    dd = Day(d)

	sY = CStr(y)
	sM = CStr(m)
	If m < 10 Then
		sM = "0" & sM
	End If
	sD = CStr(dd)
	If dd < 10 Then
		sD = "0" & sD
	End If
    DateToStandardString = sY & "-" & sM & "-" & sD
End Function

' ====================================================================
' Tests whether the difference between both dates is between 20 and 50 years
'
' d1: a Date object
' d2: a Date object
' Returns: a Boolean value
' ====================================================================
Function IsDateDifferenceOK(d1, d2)
    IsDateDifferenceOK = (d1 < d2) AND (Year(d2) > (Year(d1) + 20)) AND (Year(d2) < (Year(d1) + 50))
End Function

' ====================================================================
' Main procedure of the script
' ====================================================================
Sub Main
    ' Initialize the random number engine
    Randomize

    ' Open the file and pour into some comments
    Dim objFileSystem, objOutputFile
    Set objFileSystem = CreateObject("Scripting.fileSystemObject")
    Set objOutputFile = objFileSystem.CreateTextFile("./dates.sql", TRUE)
    objOutputFile.WriteLine("-- Fills the 'dates' database table with random dates")
    objOutputFile.WriteLine()
    objOutputFile.WriteLine("-- For performance purposes:")
    objOutputFile.WriteLine("PRAGMA synchronous = OFF;")
    objOutputFile.WriteLine()
    Dim d1, d2

    ' people.sql contains 950 random names; for each, generate a random
    ' birthday date and a random anniversary date, both separated by at
    ' least 20 to 50 years.
    For i = 1 To 950
        Do
            d1 = CDate(RandomDateInRange(CDate("1/1/1931"), CDate("12/31/1981")))
            d2 = CDate(RandomDateInRange(CDate("1/1/1951"), CDate("12/31/2001")))
        Loop Until IsDateDifferenceOK(d1, d2)

        ' "0" means birth date
        ' "1" means anniversary date
        objOutputFile.WriteLine("INSERT INTO dates (person_id, value, kind) VALUES (" & CStr(i) & ", '" & DateToStandardString(d1) & "', 0) ;")
        objOutputFile.WriteLine("INSERT INTO dates (person_id, value, kind) VALUES (" & CStr(i) & ", '" & DateToStandardString(d2) & "', 1) ;")
        objOutputFile.WriteLine()
    Next

    ' Create an index in the SQL file for faster access
    objOutputFile.WriteLine()
    objOutputFile.WriteLine("-- Create indexes for fast access")
    objOutputFile.WriteLine("CREATE INDEX dateindex ON dates(value, kind);")
    objOutputFile.WriteLine("CREATE INDEX personindex ON dates(person_id);")

    ' Close the file and cleanup memory
    objOutputFile.Close
    Set objOutputFile = Nothing
    Set objFileSystem = Nothing

    ' Report success to the user
    WScript.Echo("Finished creating the 'dates.sql' file.")
End Sub


' ====================================================================
' Entry point of the application
' ====================================================================
Main
WScript.Quit(0)
