/*!
 * \file dataaccesslayertest.h
 * Contains the interface of the test::DataAccessLayerTest class.
 */

#ifndef DATAACCESSLAYERTEST_H_
#define DATAACCESSLAYERTEST_H_

#include <cppunit/extensions/HelperMacros.h>

/*!
 * \namespace tests
 * This namespace holds the classes that derive from 
 * CppUnit::TestFixture, containing unit tests for the BirthdayCard
 * application.
 */
namespace tests
{
    /*!
     * \class DataAccessLayerTest
     * Tests the business::DataAccessLayer class.
     */
    class DataAccessLayerTest : public CppUnit::TestFixture
    {
        CPPUNIT_TEST_SUITE(tests::DataAccessLayerTest);
        CPPUNIT_TEST(testCreateDatabase);
        CPPUNIT_TEST(testCreateTable);
        CPPUNIT_TEST(testInsertion);
        CPPUNIT_TEST(testSelect);
        CPPUNIT_TEST_SUITE_END();
    
    public:
    
        /*!
         * Constructor.
         */
        DataAccessLayerTest();

        /*!
         * Virtual destructor.
         */
        virtual ~DataAccessLayerTest();
    
        /*!
         * Called by CppUnit before each test.
         */
        void setUp();

        /*!
         * Called by CppUnit after each test.
         */
        void tearDown();
        
        /*!
         * Tests the automatic creation of a database file.
         */
        void testCreateDatabase();
        
        /*!
         * Tests the creation of tables.
         */
        void testCreateTable();
        
        /*!
         * Tests the insertion of values in the table created previously.
         */
        void testInsertion();
        
        /*!
         * Tests the reading of values stored previously.
         */
        void testSelect();
        
    private:
        const std::string _filename;
    };
}

CPPUNIT_TEST_SUITE_REGISTRATION(tests::DataAccessLayerTest);

#endif /*DATAACCESSLAYERTEST_H_*/
