/*!
 * \file persontest.h
 * Contains the interface of the test::PersonTest class.
 */

#ifndef PERSONTEST_H_
#define PERSONTEST_H_

#include <cppunit/extensions/HelperMacros.h>

#ifndef PERSISTABLE_H_
#include "../business/persistable.h"
#endif

using namespace business;

/*!
 * \namespace tests
 * This namespace holds the classes that derive from 
 * CppUnit::TestFixture, containing unit tests for the BirthdayCard
 * application.
 */
namespace tests
{
    /*!
     * \class PersonTest
     * Tests the business::Person class.
     */
    class PersonTest : public CppUnit::TestFixture
    {
        CPPUNIT_TEST_SUITE(tests::PersonTest);
        CPPUNIT_TEST(testInsert);
        CPPUNIT_TEST(testSelect);
        CPPUNIT_TEST(testCopy);
        CPPUNIT_TEST(testUpdate);
        CPPUNIT_TEST(testDelete);
        CPPUNIT_TEST_SUITE_END();

    public:

        /*!
         * Constructor.
         */
        PersonTest();
        
        /*!
         * Virtual destructor.
         */
        virtual ~PersonTest();

        /*!
         * Called by CppUnit before each test.
         */
        void setUp();
        
        /*!
         * Called by CppUnit after each test.
         */
        void tearDown();

        /*!
         * Tests adding people to the database, including their
         * associated dates.
         */
        void testInsert();
        
        /*!
         * Tests getting people from the database, including their
         * associated dates.
         */
        void testSelect();
        
        /*!
         * Tests the copy constructor and the assignment operator.
         */
        void testCopy();
        
        /*!
         * Tests updating people in the database.
         */
        void testUpdate();
        
        /*!
         * Tests removing people from the database.
         */
        void testDelete();

    private:
    
        //! The ID of the test person.
        PersistableId     _personId;
        
        //! The first name of the test person.
        std::string       _firstName;
        
        //! The last name of the test person.
        std::string       _lastName;
        
        //! The filename of the SQLite database used for the tests.
        const std::string _filename;
    };
}

CPPUNIT_TEST_SUITE_REGISTRATION(tests::PersonTest);

#endif /*PERSONTEST_H_*/
