/*!
 * \file carddialog.cpp
 * Contains the implementation of the ui::CardDialog class.
 */

#include "carddialog.h"

#ifndef CONTROLLER_H_
#include "controller.h"
#endif

BEGIN_EVENT_TABLE(ui::CardDialog, wxDialog)
    EVT_PAINT(ui::CardDialog::OnPaint)
    EVT_BUTTON(ui::EventPrint, ui::CardDialog::OnPrint)
END_EVENT_TABLE()

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    CardDialog::CardDialog(const wxString& title, 
                            const wxString& event, 
                            const wxString& name)
    : wxDialog     (NULL, wxID_ANY, title, wxDefaultPosition, wxSize(600, 300),
                    wxDEFAULT_FRAME_STYLE - wxRESIZE_BORDER - wxMAXIMIZE_BOX - wxMINIMIZE_BOX)
    , wxPrintout   (title)
    , _event       (event)
    , _name        (name)
    {
        wxButton* cancel = new wxButton(this, wxID_CANCEL, wxT("Close"),
    							    wxPoint(10, 10), wxDefaultSize);
        wxButton* print = new wxButton(this, EventPrint, wxT("Print"),
    							    wxPoint(10, 40), wxDefaultSize);
    }

    void CardDialog::DrawCard(wxDC& dc)
    {
        dc.Clear();
        const wxFont font(40, wxSWISS, wxFONTSTYLE_ITALIC, wxFONTWEIGHT_NORMAL);
        const wxPen red(*wxRED, 5);
        const wxPen green(*wxGREEN, 5);
        const wxPen blue(*wxBLUE, 5);
        const wxPen black(*wxBLACK, 5);
        
        dc.SetBrush(wxBrush(*wxBLUE, wxSOLID));
        dc.SetPen(blue);
        dc.DrawCircle(180, 80, 200);
        
        dc.SetBrush(wxBrush(*wxGREEN, wxSOLID));
        dc.SetPen(green);
        dc.DrawCircle(400, 120, 100);
        
        dc.SetBrush(wxBrush(*wxRED, wxSOLID));
        dc.SetPen(red);
        dc.DrawCircle(350, 230, 50);

        dc.SetPen(black);
        dc.SetFont(font);

        wxString greeting;
        greeting << "Happy ";
        greeting << _event;
        greeting << ",";
        _name << "!";
        dc.DrawText(greeting, 50, 50);
        dc.DrawText(_name, 250, 120);
    }

    void CardDialog::OnPaint(wxPaintEvent &WXUNUSED(event))
    {
        wxPaintDC dc(this);
        DrawCard(dc);
    }

    void CardDialog::OnPrint(wxCommandEvent& WXUNUSED(event))
    {
        wxPrintData printData;
        printData.SetOrientation(wxLANDSCAPE);
        printData.SetColour(false);
        printData.SetPaperId(wxPAPER_A4);
        printData.SetQuality(wxPRINT_QUALITY_DRAFT);

        wxPrintDialogData printDialogData(printData);
        wxPrinter printer(&printDialogData);
        if (!printer.Print(this, this, true))
        {
            if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
            {
                wxMessageBox(_T("There was a problem printing."), _T("Printing"), wxOK);
            }
        }
        else
        {
            printData = printer.GetPrintDialogData().GetPrintData();
        }
    }

    bool CardDialog::OnPrintPage(int page)
    {
        wxDC *dc = GetDC();
        if (dc)
        {
            DrawCard(*dc);
            dc->SetDeviceOrigin(0, 0);
            dc->SetUserScale(8.0, 8.0);
            return true;
        }
        else
        {
            return false;
        }
    }

    bool CardDialog::OnBeginDocument(int startPage, int endPage)
    {
        if (!wxPrintout::OnBeginDocument(startPage, endPage))
        {
            return false;
        }
        return true;
    }

    void CardDialog::GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo)
    {
        *minPage = 1;
        *maxPage = 1;
        *selPageFrom = 1;
        *selPageTo = 1;
    }

    bool CardDialog::HasPage(int pageNum)
    {
        return (pageNum == 1);
    }
}
