/*!
 * \file carddialog.h
 * Contains the interface of the ui::CardDialog class.
 */

#ifndef CARDDIALOG_H_
#define CARDDIALOG_H_

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/print.h>

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    class Controller;

    /*!
     * \class CardDialog
     * This class is a printable dialog, that represents a 
     * card to be sent to someone that has a special event today.
     */
    class CardDialog : public wxDialog, public wxPrintout
    {
    public:
        CardDialog(const wxString& title, 
                    const wxString& event, 
                    const wxString& name);

        void OnPaint(wxPaintEvent& event);
        void OnPrint(wxCommandEvent& event);

        bool OnPrintPage(int page);
        bool HasPage(int page);
        bool OnBeginDocument(int startPage, int endPage);
        void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);

    private:

        void DrawCard(wxDC& dc);

    private:

        wxString     _event;
        wxString     _name;

        DECLARE_EVENT_TABLE()
    };

    // IDs for the controls and the menu commands
    enum
    {
        EventPrint
    };
}

#endif /*CARDDIALOG_H_*/
