/*!
 * \file datedialog.cpp
 * Contains the implementation of the ui::DateDialog class.
 */

#include "datedialog.h"

#ifndef CONTROLLER_H_
#include "controller.h"
#endif

BEGIN_EVENT_TABLE(ui::DateDialog, wxDialog)
    EVT_BUTTON(ui::OK, ui::DateDialog::OnOK)
END_EVENT_TABLE()

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    DateDialog::DateDialog(Controller& controller, bool editing)
    : wxDialog     (NULL, wxID_ANY, wxT("Date Chooser"), wxDefaultPosition, wxSize(260, 300),
                    wxDEFAULT_FRAME_STYLE - wxRESIZE_BORDER - wxMAXIMIZE_BOX - wxMINIMIZE_BOX)
    , _editing     (editing)
    , _choice      (0)
    , _calendar    (0)
    , _controller  (controller)
    {
        wxArrayString choices;
        choices.Add("Birthday");
        choices.Add("Anniversary");
        _choice = new wxListBox(this, wxID_ANY, wxPoint(20, 10), wxSize(100, 40), choices);
        _calendar = new wxCalendarCtrl(this, wxID_ANY,
                                        wxDefaultDateTime,
                                        wxPoint(20, 60),
                                        wxDefaultSize,
                                        wxRAISED_BORDER |
                                        wxCAL_SUNDAY_FIRST);
        _calendar->SetHeaderColours(*wxBLUE, *wxWHITE);
        _calendar->EnableYearChange();

        wxButton* buttonCancel = new wxButton(this, wxID_CANCEL, wxT("Cancel"),
    							    wxPoint(20, 240), wxDefaultSize);
        wxButton* buttonOK = new wxButton(this, OK, wxT("OK"),
    							    wxPoint(120, 240), wxDefaultSize);
        buttonOK->SetDefault();

        if (_editing)
        {
            _controller.updateCalendar(_calendar);
            _controller.updateDateKindControl(_choice);
        }
    }

    void DateDialog::OnOK(wxCommandEvent& WXUNUSED(event))
    {
        wxDateTime chosen = _calendar->GetDate();
        int kind = _choice->GetSelection();
        if (wxNOT_FOUND == kind)
        {
            kind = -1;
        }
        _controller.saveDate(chosen, kind, _editing);
        EndModal(wxID_OK);
    }
}
