/*!
 * \file main.h
 * Contains the interface of the ui::Main class.
 */

#ifndef MAIN_H_
#define MAIN_H_

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/listctrl.h>

#ifndef CONTROLLER_H_
#include "controller.h"
#endif

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    /*!
     * \class Main
     * Represents the main window of the application. It is
     * launched by the ui::BirthdayCard class upon application launch
     * and uses a ui::Controller instance to delegate all 
     * persistence tasks.
     */
    class Main : public wxFrame
    {
    public:
        Main(const wxString& title);
        
        // event handlers (these functions should _not_ be virtual)
        void OnFileQuit(wxCommandEvent& event);

        void OnHelpAbout(wxCommandEvent& event);

        void OnPersonNew(wxCommandEvent& event);
        void OnPersonEdit(wxCommandEvent& event);
        void OnPersonDelete(wxCommandEvent& event);
        void OnPersonDeleteAll(wxCommandEvent& event);
        void OnPersonSelect(wxListEvent& event);
        void OnDateSelect(wxListEvent& event);
        void OnShowTodayEvents(wxCommandEvent& event);
        void OnSave(wxCommandEvent& event);
        void OnDeleteDate(wxCommandEvent& event);
        void OnFirstNameEnter(wxCommandEvent& event);
        void OnAddDate(wxCommandEvent& event);
        void OnEditDate(wxCommandEvent& event);
        
    private:

        void CreateMenu();
        void CreatePeopleList();
        void CreateDatesList();
        void CreatePanel();
        void CreateMainPanel();
        void CreateButtons();

    private:
        wxListCtrl*     _peopleList;
        wxListCtrl*     _datesList;
        wxPanel*        _innerPanel;
        wxPanel*        _mainPanel;
        wxTextCtrl*     _firstName;
        wxTextCtrl*     _lastName;
        wxCalendarCtrl* _calendar;
        wxButton*       _buttonSave;
        wxButton*       _buttonEdit;
        wxButton*       _buttonAddDate;
        wxButton*       _buttonDeleteDate;
        wxButton*       _buttonEditDate;
        wxButton*       _buttonShowTodayEvents;
        Controller      _controller;

        DECLARE_EVENT_TABLE()
    };

    // IDs for the controls and the menu commands
    enum
    {
        // menu items
        FileQuit = wxID_EXIT,
        
        PersonNew = 100,
        PersonEdit,
        PersonDelete,
        PersonDeleteAll,
        FileShowTodayEvents,
        PeopleListbox,
        PeopleDatebox,
        SaveButton,
        EditButton,
        TodayEventsButton,
        AddDateButton,
        EditDateButton,
        DeleteDateButton,
        FirstNameTextCtrl,
        LastNameTextCtrl,

        // it is important for the id corresponding to the "About" command to have
        // this standard value as otherwise it won't be handled properly under Mac
        // (where it is special and put into the "Apple" menu)
        HelpAbout = wxID_ABOUT
    };
}

#endif /*MAIN_H_*/
