/*!
 * \file todayeventsdialog.cpp
 * Contains the implementation of the ui::TodayEventsDialog class.
 */

#include "todayeventsdialog.h"

#ifndef CONTROLLER_H_
#include "controller.h"
#endif
#ifndef CARDDIALOG_H_
#include "carddialog.h"
#endif

BEGIN_EVENT_TABLE(ui::TodayEventsDialog, wxDialog)
    EVT_LIST_ITEM_SELECTED(ui::EventsListbox, ui::TodayEventsDialog::OnEventSelect)
END_EVENT_TABLE()

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    TodayEventsDialog::TodayEventsDialog(const wxString& title, Controller& controller)
    : wxDialog(NULL, wxID_ANY, title, wxDefaultPosition, wxSize(300, 400),
                wxDEFAULT_FRAME_STYLE - wxRESIZE_BORDER - wxMAXIMIZE_BOX - wxMINIMIZE_BOX)
    , _eventsList  (0)
    , _controller  (controller)
    {
        wxStaticText* instructions = new wxStaticText(this, wxID_ANY, 
            wxT("Select any event to see its card:"), wxPoint(15, 15), wxSize(260, 30));
        _eventsList = new wxListCtrl(this, EventsListbox,
                            wxPoint(15, 45), wxSize(260, 285),
                            wxLC_REPORT + wxLC_SINGLE_SEL);

        _eventsList->InsertColumn(0, wxString("People names"),
    						    wxLIST_FORMAT_LEFT, 128);
        _eventsList->InsertColumn(1, wxString("Event"),
    						    wxLIST_FORMAT_LEFT, 128);
        _controller.fillEventsList(_eventsList);

        wxButton* cancel = new wxButton(this, wxID_CANCEL, wxT("Close"),
    							    wxPoint(15, 340), wxDefaultSize);
    }

    void TodayEventsDialog::OnEventSelect(wxListEvent& event)
    {
        long personId = event.GetData();
        wxString eventName = _controller.getEventForPersonId(personId);
        wxString name = _controller.getNameForCard(personId);
        wxString title = "Card for ";
        title << name;
        CardDialog card(title, eventName, name);
        card.Centre(wxBOTH);
        card.ShowModal();
    }
}
